/*****************************************************
CodeWizardAVR V1.25.8 Professional

Project : Laser Controller
Version : 1
Author  : Sergey Kozlov

Chip type           : ATmega48
Clock frequency     : 16,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 128
*****************************************************/

// CodeVisionAVR C Compiler
// (C) 1998-2004 Pavel Haiduc, HP InfoTech S.R.L.

// I/O registers definitions for the ATmega48(V)


#pragma used+
#pragma used+
sfrb PINB=3;
sfrb DDRB=4;
sfrb PORTB=5;
sfrb PINC=6;
sfrb DDRC=7;
sfrb PORTC=8;
sfrb PIND=9;
sfrb DDRD=0xa;
sfrb PORTD=0xb;
sfrb TIFR0=0x15;
sfrb TIFR1=0x16;
sfrb TIFR2=0x17;
sfrb PCIFR=0x1b;
sfrb EIFR=0x1c;
sfrb EIMSK=0x1d;
sfrb GPIOR0=0x1e;
sfrb EECR=0x1f;
sfrb EEDR=0x20;
sfrb EEARL=0x21;
sfrb EEARH=0x22;
sfrw EEAR=0x21;   // 16 bit access
sfrb GTCCR=0x23;
sfrb TCCR0A=0x24;
sfrb TCCR0B=0x25;
sfrb TCNT0=0x26;
sfrb OCR0A=0x27;
sfrb OCR0B=0x28;
sfrb GPIOR1=0x2a;
sfrb GPIOR2=0x2b;
sfrb SPCR=0x2c;
sfrb SPSR=0x2d;
sfrb SPDR=0x2e;
sfrb ACSR=0x30;
sfrb MONDR=0x31;
sfrb SMCR=0x33;
sfrb MCUSR=0x34;
sfrb MCUCR=0x35;
sfrb SPMCSR=0x37;
sfrb SPL=0x3d;
sfrb SPH=0x3e;
sfrb SREG=0x3f;
#pragma used-


// Interrupt vectors definitions


// Needed by the power management functions (sleep.h)
#asm
	#ifndef __SLEEP_DEFINED__
	#define __SLEEP_DEFINED__
	.EQU __se_bit=0x01
	.EQU __sm_mask=0x0E
	.EQU __sm_adc_noise_red=0x02
	.EQU __sm_powerdown=0x04
	.EQU __sm_powersave=0x06
	.EQU __sm_standby=0x0C
	.SET power_ctrl_reg=smcr
	#endif
#endasm


// CodeVisionAVR C Compiler
// (C) 1998-2000 Pavel Haiduc, HP InfoTech S.R.L.


#pragma used+
#pragma used+

void delay_us(unsigned int n);
void delay_ms(unsigned int n);

#pragma used-

/*
CodeVisionAVR C Compiler
Prototypes for mathematical functions

Portions (C) 1998-2001 Pavel Haiduc, HP InfoTech S.R.L.
Portions (C) 2000-2001 Yuri G. Salov
*/



#pragma used+
#pragma used+

unsigned char cabs(signed char x);
unsigned int abs(int x);
unsigned long labs(long x);
float fabs(float x);
signed char cmax(signed char a,signed char b);
int max(int a,int b);
long lmax(long a,long b);
float fmax(float a,float b);
signed char cmin(signed char a,signed char b);
int min(int a,int b);
long lmin(long a,long b);
float fmin(float a,float b);
signed char csign(signed char x);
signed char sign(int x);
signed char lsign(long x);
signed char fsign(float x);
unsigned char isqrt(unsigned int x);
unsigned int lsqrt(unsigned long x);
float sqrt(float x);
float floor(float x);
float ceil(float x);
float fmod(float x,float y);
float modf(float x,float *ipart);
float ldexp(float x,int expon);
float frexp(float x,int *expon);
float exp(float x);
float log(float x);
float log10(float x);
float pow(float x,float y);
float sin(float x);
float cos(float x);
float tan(float x);
float sinh(float x);
float cosh(float x);
float tanh(float x);
float asin(float x);
float acos(float x);
float atan(float x);
float atan2(float y,float x);

#pragma used-
#pragma library math.lib




//Fuctions' prototypes
char ee_read(unsigned int);
char twi_read(char);
void twi_write(char);
void twi_start(void);
void twi_stop(void); 
void out(char,char,char); 
void circle(float,char);   
void effect1(char);
void effect2(char);
void effect3(void);
void effect4(void);
void effect5(void);  
//

//Global variables declaration
char next=0;
char loop=0;
char increment=1;
char repeat;
//
char twi_read(char ACK)
{ 
        if(ACK) (*(unsigned char *) 0xbc)=128|64|4 ;
        else (*(unsigned char *) 0xbc)=128|4 ;
        while(!((*(unsigned char *) 0xbc)&(128)));
        return (*(unsigned char *) 0xbb);         
}
void twi_write(char data)
{
        (*(unsigned char *) 0xbb)=data;
        (*(unsigned char *) 0xbc)=128|4 ;
        while(!((*(unsigned char *) 0xbc)&(128)));
}   
void twi_start(void)
{
        (*(unsigned char *) 0xbc)=128|32|4 ;
        while(!((*(unsigned char *) 0xbc)&(128)));
}
void twi_stop(void)
{ 
        (*(unsigned char *) 0xbc)=128|4 |16;
}
char ee_read(unsigned int adress)
{ 
        char data;
        twi_start();
        twi_write(0xa0);
        twi_write((unsigned char)(adress>>8));
        twi_write((unsigned char)(adress&255));
        twi_start();
        twi_write(0xa0|1);
        data=twi_read(0);
        twi_stop();
        return data;       
}  


void read_picture(void)
{
        char x,y,z;
        unsigned int i;
        unsigned int counter,offset;  
        unsigned int points_number;
        char frames_number,f;//,r;
        char frame_repeats;
       
        frames_number=ee_read(0);
        frame_repeats=ee_read(1);

        while(1)
        {
        counter=1; 
        for(f=1;f<=frames_number;f++)
        {
                points_number=ee_read(++counter);
                points_number=points_number<<8;
                points_number|=ee_read(++counter);  
                ++counter;
                for(repeat=0;repeat<frame_repeats;repeat+=increment)
                {  
                        #asm("wdr");
                        offset=counter;
                        /////////////////////////////
                        twi_start();
                        twi_write(0xa0);
                        twi_write((unsigned char)(counter>>8));
                        twi_write((unsigned char)(counter));//&255
                        twi_start();
                        twi_write(0xa0|1);
                        ////////////////////////////        
                        for(i=1;i<=points_number;i++)
                        {      
                                x=twi_read(1);
                                y=twi_read(1);
                                if(i!=points_number) z=twi_read(1);
                                else z=twi_read(0); 
                                out(x,y,z);
                                if(points_number<50) delay_us(200);
                                else if(points_number<150) delay_us(100);
                                else if(points_number<200)delay_us(50);
                                offset+=3;
                        }
                        twi_stop();              
                }
                counter=offset;
                --counter;
        }
        }
}

void select_effect(void)
{
ef1:
        effect1(0); 
        if(loop&(!next)) goto ef1;
ef2:
        effect2(0);
        //effect2(1);       
        if(loop&(!next)) goto ef2;
ef3:
        effect1(1);
        if(loop&(!next)) goto ef3; 
ef4:    
        effect2(1);
        if(loop&(!next)) goto ef4;
ef5:
        effect3();
        if(loop&(!next)) goto ef5;

ef6:
        effect5();
        if(loop&(!next)) goto ef6;
ef7:
        //effect5(); 
        effect4(); 
        if(loop&(!next)) goto ef7;
}

void circle(float step,char delay)
{
        float angle;
        unsigned char xcoord;
        unsigned char ycoord;
        for(angle=0;angle<=6.283185;angle+=step) //step=0.1
        {
                xcoord=(char)(cos(angle)*127);
                ycoord=(char)(sin(angle)*127); 
                out(xcoord+127,ycoord+127,0);
                if(delay) delay_us(700);
                PORTD.6=1;
                if(delay) delay_ms(5);

        };
}

void effect1(char plane)//poyavleniye luchey
{  
        char i;
        char j;
        char counter;
        next=0;
                for(i=0;i<=4;i++)
                { 
                        for(j=0;j<=100;j++)
                        { 
                                counter=0;
                                delay_ms(2);
                                while(counter<=i)
                                {
                                        if(plane) out(counter*51,127,1);
                                        else      out(127,counter*51,1);
                                        delay_ms(5);
                                        ++counter;
                                };
                                if(plane) out(0,127,0);
                                else      out(127,0,0);
                        };
                        #asm("wdr"); 
                        if(next) return;
                };
        
}

void effect2(char plane)//dvijeniye luchey
{
        char i;
        char j;
        char counter;
        next=0;
        for(j=0;j<=3;j++)
        {                      
        for(i=127;i<=254;i+=2)
        {
                                counter=0;
                                delay_ms(2);
                                while(counter<=4)
                                {
                                        if(plane) out(counter*51,i,1);
                                        else      out(i,counter*51,1);
                                        delay_ms(5);
                                        ++counter;
                                };
                                if(plane) out(0,i,0);
                                else      out(i,0,0);
                        #asm("wdr"); 
                        if(next) return;
        }; 
        for(i=254;i>=1;i-=2)
        {
                                counter=0;
                                delay_ms(2);
                                while(counter<=4)
                                {
                                        if(plane) out(counter*51,i,1);
                                        else      out(i,counter*51,1);
                                        delay_ms(5);
                                        ++counter;
                                };
                                if(plane) out(0,i,0);
                                else      out(i,0,0);
                        #asm("wdr"); 
                        if(next) return;
        };
        for(i=1;i<=127;i+=2)
        {
                                counter=0;
                                delay_ms(2);
                                while(counter<=4)
                                {
                                        if(plane) out(counter*51,i,1);
                                        else      out(i,counter*51,1);
                                        delay_ms(5);
                                        ++counter;
                                };
                                if(plane) out(0,i,0);
                                else      out(i,0,0);
                        #asm("wdr"); 
                        if(next) return;
        };
        };

}

void effect3(void)//vrashyayushiesya luchi 
{
        int i;
        float step;
        next=0;
        step=1.1;
        for(i=0;i<=1200;i++)
        { 
                circle(step,1);
                if(next) return;
                step+=0.0005;
        };
}
 
void effect4(void)//tonnel'
{
        int i;
        for(i=0;i<=800;i++)
        { 
                circle(0.2,0);
                if(next) return;
                #asm("wdr");
        };
}
 
void effect5(void)//ploskosti 
{
        char i;
        char j; 
        char k;
        char l;
        bit plane;
        next=0;
        for(l=1;l<=4;l++)
        {
        for(i=127;i<=254;i++)
        { 
                k=255-i;
                for(j=k;j<=i;j++)
                {       
                        if(plane)out(j,j,1);
                        else out(255-j,j,1);
                        if(j==k)
                        { 
                                PORTD.6=0;
                                delay_ms(4);
                                PORTD.6=1;                        
                        };
                        delay_us(80);
                };
                if(next) return;
        };         
        #asm("wdr");                 
        for(i=254;i>=127;i--)
        {
                k=255-i; 
                for(j=k;j<=i;j++)
                { 
                        if(plane)out(j,j,1);
                        else out(255-j,j,1);
                        if(j==k)
                        { 
                                PORTD.6=0;
                                delay_ms(4);
                                PORTD.6=1;                        
                        };
                        delay_us(80);
                };
                if(next) return;
                PORTD.6=0;
                delay_us(700);
                PORTD.6=1;
        };
        #asm("wdr");
        plane=~plane;
        };
}
void out(char x, char y, char blanking)
{
        PORTB=x;
        PORTC=4|(x>>6);
        PORTC.2=0; 
        PORTC.2=1; 
        PORTB=y;
        PORTC=4|(y>>6);
        PORTD.7=0;
        PORTD.7=1;
        PORTD.6=blanking;
        return;
        
}

void test(void)//Laser test patterns
{
        unsigned char i=0;
        unsigned char counter=0;
        #asm("sei");
        while(1)
        {       
                #asm("wdr");
                if(next==1)
                { 
                        ++counter;
                        if(counter>3) counter=0;
                        next=0;
                };
                if(counter==0)
                {
                        for(i=0;i<=254;i++)
                        { 
                                if((i<85)||(i>170))
                                {
                                        out(i,0,1);
                                }
                                else
                                {
                                        out(i,255,1);
                                };
                                delay_us(100);
                        };
                }
                else if(counter==1)
                { 
                        for(i=0;i<=254;i++)
                        { 
                                if((i<85)||(i>170)) out(0,i,1);
                                else out(255,i,1);
                                delay_us(100);
                        };
                
                }
                else if(counter==2)
                { 
                        for(i=0;i<=254;i++)
                        { 
                                out(0,i,1);
                                delay_us(10);
                        };                    
                        for(i=0;i<=254;i++)
                        { 
                                out(i,255,1);
                                delay_us(10);
                        };                    
                        for(i=254;i>=1;i--)
                        { 
                                out(255,i,1);
                                delay_us(10);
                        };                    
                        for(i=254;i>=1;i--)
                        { 
                                out(i,0,1);
                                delay_us(10);
                        };
                }
                else
                { ///>>>circle here
                        circle(0.5,0);
                }; 

        };
}

// External Interrupt 0 service routine
interrupt [2] void ext_int0_isr(void)
{
// Place your code here
                next=1;
                repeat=200;

}

// External Interrupt 1 service routine
interrupt [3] void ext_int1_isr(void)
{
// Place your code here
        if(loop)
        {
                loop=0;
                increment=1;
                PORTD.5=0;
        }
        else
        { 
                loop=1;
                increment=0;
                PORTD.5=1;
        }; 
}

// Declare your global variables here

void main(void)
{
// Declare your local variables here
char i;
// Crystal Oscillator division factor: 1
#pragma optsize-
(*(unsigned char *) 0x61)=0x80;
(*(unsigned char *) 0x61)=0x00;
#pragma optsize+

// Input/Output Ports initialization
// Port B initialization
// Func7=In Func6=In Func5=Out Func4=Out Func3=Out Func2=Out Func1=Out Func0=Out 
// State7=T State6=T State5=0 State4=0 State3=0 State2=0 State1=0 State0=0 
PORTB=0x00;
DDRB=0x3F;

// Port C initialization
// Func6=In Func5=In Func4=In Func3=In Func2=Out Func1=Out Func0=Out 
// State6=T State5=T State4=T State3=T State2=0 State1=0 State0=0 
PORTC=56;//PC3,4,5 pullup
DDRC=0x07;

// Port D initialization
// Func7=Out Func6=Out Func5=Out Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=0 State6=0 State5=0 State4=T State3=T State2=T State1=T State0=T 
PORTD=0x00;
DDRD=0xE0;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: Timer 0 Stopped
// Mode: Normal top=FFh
// OC0A output: Disconnected
// OC0B output: Disconnected
TCCR0A=0x00;
TCCR0B=0x00;
TCNT0=0x00;
OCR0A=0x00;
OCR0B=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer 1 Stopped
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
(*(unsigned char *) 0x80)=0x00;
(*(unsigned char *) 0x81)=0x00;
(*(unsigned char *) 0x85)=0x00;
(*(unsigned char *) 0x84)=0x00;
(*(unsigned char *) 0x87)=0x00;
(*(unsigned char *) 0x86)=0x00;
(*(unsigned char *) 0x89)=0x00;
(*(unsigned char *) 0x88)=0x00;
(*(unsigned char *) 0x8b)=0x00;
(*(unsigned char *) 0x8a)=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer 2 Stopped
// Mode: Normal top=FFh
// OC2A output: Disconnected
// OC2B output: Disconnected
(*(unsigned char *) 0xb6)=0x00;
(*(unsigned char *) 0xb0)=0x00;
(*(unsigned char *) 0xb1)=0x00;
(*(unsigned char *) 0xb2)=0x00;
(*(unsigned char *) 0xb3)=0x00;
(*(unsigned char *) 0xb4)=0x00;

// External Interrupt(s) initialization
// INT0: On
// INT0 Mode: Falling Edge
// INT1: On
// INT1 Mode: Falling Edge
// Interrupt on any change on pins PCINT0-7: Off
// Interrupt on any change on pins PCINT8-14: Off
// Interrupt on any change on pins PCINT16-23: Off
(*(unsigned char *) 0x69)=0x0A;
EIMSK=0x03;
EIFR=0x03;
(*(unsigned char *) 0x68)=0x00;

// Timer/Counter 0 Interrupt(s) initialization
(*(unsigned char *) 0x6e)=0x00;
// Timer/Counter 1 Interrupt(s) initialization
(*(unsigned char *) 0x6f)=0x00;
// Timer/Counter 2 Interrupt(s) initialization
(*(unsigned char *) 0x70)=0x00;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
(*(unsigned char *) 0x7b)=0x00;

// 2 Wire Bus initialization
// Generate Acknowledge Pulse: On
// 2 Wire Bus Slave Address: 0h
// General Call Recognition: Off
// Bit Rate: 400,000 kHz
(*(unsigned char *) 0xb9)=0x00;
(*(unsigned char *) 0xb8)=0x0C;
(*(unsigned char *) 0xba)=0x00;
(*(unsigned char *) 0xbc)=0x44;

// Watchdog Timer initialization
// Watchdog Timer Prescaler: OSC/1024k
// Watchdog Timer interrupt: Off
#pragma optsize-
//#asm("wdr")
(*(unsigned char *) 0x60)=0x39;
(*(unsigned char *) 0x60)=0x29;
#pragma optsize+

//delay_ms(1000);//vremya ustanovki vhodov
for(i=0;i<=20;i++)
{
        if((~PIND.2)&(~PIND.3)) test();//vhod v testoviy rejim
        delay_ms(100);
        #asm("wdr");
};
while(PIND.2&PIND.3) #asm("wdr");//jdat' poka ne najata kakaya-nibud' knopka

// Global enable interrupts
#asm("sei")
//test();
while (1)
      {
      // Place your code here 
      if(PINC.3==1)
        select_effect();
      else
        read_picture();
      };
}
